@@ -371,6 +371,37 @@ QUnit.test( "multiple", function( assert ) {
 		[ "h2", "firstp", "ap", "sndp", "en", "sap", "first" ] );
 } );
 
+( function() {
+	var supportsValidPseudo;
+	try {
+		supportsValidPseudo = document.querySelectorAll( "input:valid" ) &&
+
+			// Support: Firefox 3.6 only
+			// Firefox 3.6 doesn't support the `:valid` pseudo-class, but it also
+			// doesn't throw when a `qSA` call includes it.
+			!/firefox\/3\.6/i.test( navigator.userAgent );
+	} catch ( e ) {
+		supportsValidPseudo = false;
+	}
+
+	if ( supportsValidPseudo ) {
+		QUnit.test( "multiple, only supported natively (jquery/jquery#5177)", function( assert ) {
+			assert.expect( 4 );
+
+			jQuery( "#qunit-fixture" ).prepend( "<h2 id='h2'/>" );
+
+			t( "Comma Support", "#qunit-fixture #search:valid, #qunit-fixture p",
+				[ "firstp", "ap", "sndp", "en", "sap", "first", "search" ] );
+			t( "Comma Support", "#qunit-fixture #search:valid , #qunit-fixture p",
+				[ "firstp", "ap", "sndp", "en", "sap", "first", "search" ] );
+			t( "Comma Support", "#qunit-fixture #search:valid,#qunit-fixture p",
+				[ "firstp", "ap", "sndp", "en", "sap", "first", "search" ] );
+			t( "Comma Support", "#qunit-fixture #search:valid\t,\r#qunit-fixture p\n",
+				[ "firstp", "ap", "sndp", "en", "sap", "first", "search" ] );
+		} );
+	}
+} )();
+
 QUnit.test( "child and adjacent", function( assert ) {
 	assert.expect( 43 );
 
@@ -578,7 +609,7 @@ QUnit.test( "attributes - special characters", function( assert ) {
 	var attrbad,
 		div = document.createElement( "div" );
 
-	// #3279
+	// #3729
 	div.innerHTML = "<div id='foo' xml:test='something'></div>";
 	assert.deepEqual( Sizzle( "[xml\\:test]", div ),
 		[ div.firstChild ],
@@ -842,11 +873,19 @@ QUnit.test( "pseudo - nth-last-of-type", function( assert ) {
 } );
 
 QUnit.test( "pseudo - has", function( assert ) {
-	assert.expect( 3 );
+	assert.expect( 4 );
 
 	t( "Basic test", "p:has(a)", [ "firstp", "ap", "en", "sap" ] );
 	t( "Basic test (irrelevant whitespace)", "p:has( a )", [ "firstp", "ap", "en", "sap" ] );
 	t( "Nested with overlapping candidates", "#qunit-fixture div:has(div:has(div:not([id])))", [ "moretests", "t2037" ] );
+
+	// Support: Safari 15.4+, Chrome 107+?
+	// `qSA` in Safari throws for `:has()` with only unsupported arguments
+	// but if you add a supported arg to the list, it will run and just potentially
+	// return no results. Make sure this is accounted for.
+	t( "Nested with list arguments",
+		"#qunit-fixture div:has(faketag, div:has(faketag, div:not([id])))",
+		[ "moretests", "t2037" ] );
 } );
 
 QUnit.test( "pseudo - contains", function( assert ) {
